@@ -8,7 +8,7 @@ use extended_primitives::Buffer;
 use futures::channel::mpsc::{UnboundedReceiver, UnboundedSender};
 use futures::SinkExt;
 use futures::StreamExt;
-use log::{debug, info, trace, warn};
+use log::{debug, trace, warn};
 use rand::prelude::*;
 use serde::Serialize;
 use serde_json::json;
@@ -32,12 +32,14 @@ pub enum ConnectionState {
 //completed.
 #[derive(Debug)]
 pub struct Connection<State> {
+    pub session_id: u32,
     pub sender: Arc<Mutex<UnboundedSender<String>>>,
     pub upstream_sender: Arc<Mutex<UnboundedSender<String>>>,
     pub upstream_receiver:
         Arc<Mutex<UnboundedReceiver<serde_json::map::Map<String, serde_json::Value>>>>,
     pub authorized: Arc<Mutex<bool>>,
     pub subscribed: Arc<Mutex<bool>>,
+    pub agent: Arc<Mutex<bool>>,
     pub session_start: SystemTime,
     pub connection_state: Mutex<ConnectionState>,
     pub difficulty: Arc<Mutex<f64>>,
@@ -47,8 +49,6 @@ pub struct Connection<State> {
 
     //@todo review the rest of these variables here.
     pub id: Uuid,
-    pub subscriber_id: Arc<Mutex<String>>,
-    pub miner_info: Arc<RwLock<MinerInfo>>,
     //Possibly pull these out into their own var.
     //Makes it easier to operate on them.
     pub job_stats: Arc<Mutex<JobStats>>,
@@ -59,31 +59,10 @@ pub struct Connection<State> {
     pub state: Arc<Mutex<State>>,
     pub stop_source: Arc<Mutex<Option<StopSource>>>,
     pub stop_token: StopToken,
-    pub worker_list: Arc<Mutex<HashMap<u16, WorkerInfo>>>,
-}
-
-//@todo review this. I'm not sure if this is the best strategy here, but I'd like for the ability
-//to pass some information to the implementer of traits. I.E. For logging in, the implementer
-//probably wants the IP of the connection that is attempting to log in, so that they can rate limit
-//or ban. Further, for share submitting and other features, the implementer probably wants the same
-//ability. Rather than pass the entire stream into the trait functions, I figured we should build a
-//struct that holds some information about the miner that can be edited and passed into those
-//functions.
-#[derive(Clone, Debug)]
-pub struct MinerInfo {
-    pub ip: IpAddr,
-    pub auth: Option<MinerAuth>,
-    pub id: Option<Uuid>,
-    pub sid: Option<String>,
-    pub job_stats: Option<MinerJobStats>,
-    pub name: Option<String>,
-}
-
-#[derive(Clone, Debug)]
-pub struct MinerAuth {
-    pub id: String,
-    pub username: String,
-    pub client: String,
+    //@todo I'm starting to think we pull this out into our own file to make it a lot easier to do
+    //things.
+    //impl functions directly onto agentworker rather than copy paste in here
+    pub worker_list: Arc<Mutex<HashMap<u16, AgentWorkerInfo>>>,
 }
 
 #[derive(Clone, Debug)]
@@ -133,8 +112,22 @@ pub struct WorkerInfo {
     pub id: Uuid,
 }
 
+#[derive(Debug, Clone)]
+pub struct AgentWorkerInfo {
+    pub client: Option<String>,
+    pub name: Option<String>,
+    pub sid: Option<Buffer>,
+    //@todo these all seem like an extra waste of energy.
+    pub account_id: i32,
+    pub miner_account: Option<i32>,
+    pub id: Uuid,
+    pub stats: Arc<Mutex<MinerStats>>,
+    pub job_stats: Arc<Mutex<JobStats>>,
+}
+
 impl<State: Clone + Send + Sync + 'static> Connection<State> {
     pub fn new(
+        session_id: u32,
         addr: SocketAddr,
         sender: UnboundedSender<String>,
         upstream_sender: UnboundedSender<String>,
@@ -150,16 +143,6 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
 
         debug!("Accepting new miner. ID: {}", &id);
 
-        let info = MinerInfo {
-            ip: addr.ip(),
-            auth: None,
-            id: None,
-            sid: None,
-            job_stats: None,
-            //@todo delete this as it's in WorkerInfo.
-            name: None,
-        };
-
         let options = MinerOptions {
             retarget_time: var_diff_config.retarget_time,
             target_time: var_diff_config.target_time,
@@ -196,6 +179,7 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
 
         //@todo clean up the comments here.
         Connection {
+            session_id,
             id,
             // write_half: Arc::new(Mutex::new(wh)),
             // read_half: Arc::new(Mutex::new(rh)),
@@ -206,10 +190,9 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
             session_start: SystemTime::now(),
             connection_state: Mutex::new(ConnectionState::Connected),
             subscribed: Arc::new(Mutex::new(false)),
+            agent: Arc::new(Mutex::new(false)),
             //@todo this should be passed in.
             difficulty: Arc::new(Mutex::new(initial_difficulty)),
-            subscriber_id: Arc::new(Mutex::new(String::new())),
-            miner_info: Arc::new(RwLock::new(info)),
             next_difficulty: Arc::new(Mutex::new(None)),
             job_stats: Arc::new(Mutex::new(job_stats)),
             options: Arc::new(options),
@@ -225,65 +208,10 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
         }
     }
 
-    pub async fn info(&self) -> MinerInfo {
-        self.miner_info.read().await.clone()
-    }
-
     pub async fn is_disconnected(&self) -> bool {
         *self.connection_state.lock().await == ConnectionState::Disconnect
     }
 
-    //pub async fn next_message(
-    //    &self,
-    //) -> Result<(String, serde_json::map::Map<String, serde_json::Value>)> {
-    //    //I don't actually think this has to loop here.
-    //    loop {
-    //        let mut stream = self.read_half.lock().await;
-
-    //        let mut buf = String::new();
-    //        let num_bytes = stream.read_line(&mut buf).await?;
-
-    //        if num_bytes == 0 {
-    //            self.shutdown().await?;
-    //            return Err(Error::StreamClosed);
-    //        }
-
-    //        if !buf.is_empty() {
-    //            //@smells
-    //            buf = buf.trim().to_owned();
-    //            debug!("Received Message: {}", &buf);
-    //            let msg: Map<String, Value> = serde_json::from_str(&buf)?;
-
-    //            let method = if msg.contains_key("method") {
-    //                match msg.get("method") {
-    //                    Some(method) => method.as_str(),
-    //                    //@todo need better stratum erroring here.
-    //                    None => return Err(Error::MethodDoesntExist),
-    //                }
-    //            } else if msg.contains_key("messsage") {
-    //                match msg.get("message") {
-    //                    Some(method) => method.as_str(),
-    //                    //@todo need better stratum erroring here.
-    //                    None => return Err(Error::MethodDoesntExist),
-    //                }
-    //            } else {
-    //                return Err(Error::MethodDoesntExist);
-    //            };
-
-    //            if let Some(method_string) = method {
-    //                //Mark the sender as active as we received a message.
-    //                //We only mark them as active if the message/method was valid
-    //                self.stats.lock().await.last_active = Utc::now().naive_utc();
-
-    //                return Ok((method_string.to_owned(), msg));
-    //            } else {
-    //                //@todo improper format
-    //                return Err(Error::MethodDoesntExist);
-    //            }
-    //        };
-    //    }
-    //}
-
     //@todo we have disabled last_active for now... Need to reimplement this desperately.
     pub async fn send<T: Serialize>(&self, message: T) -> Result<()> {
         //let last_active = self.stats.lock().await.last_active;
@@ -497,10 +425,6 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
         self.stats.lock().await.clone()
     }
 
-    pub async fn ip(&self) -> IpAddr {
-        self.info().await.ip
-    }
-
     pub fn id(&self) -> Uuid {
         self.id
     }
@@ -530,7 +454,7 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
         worker_name: &str,
         worker_id: Uuid,
     ) {
-        let worker = WorkerInfo {
+        let worker = AgentWorkerInfo {
             client: Some(client_agent.to_owned()),
             name: Some(worker_name.to_owned()),
             //@todo I suppose this is technically session ID
@@ -539,6 +463,19 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
             account_id: self.get_account_id().await,
             miner_account: self.get_miner_account().await,
             id: worker_id,
+            stats: Arc::new(Mutex::new(MinerStats {
+                accepted_shares: 0,
+                rejected_shares: 0,
+                last_active: Utc::now().naive_utc(),
+            })),
+            job_stats: Arc::new(Mutex::new(JobStats {
+                last_timestamp: Utc::now().naive_utc().timestamp(),
+                last_share_timestamp: 0,
+                last_retarget: Utc::now().naive_utc().timestamp() - self.options.retarget_time / 2,
+                times: Vec::new(),
+                current_difficulty: self.difficulty.lock().await.clone(),
+                job_difficulty: 0.0,
+            })),
         };
 
         self.worker_list.lock().await.insert(session_id, worker);
@@ -550,7 +487,12 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
         self.worker_list.lock().await.remove(&session_id);
     }
 
-    pub async fn get_worker_by_session_id(&self, session_id: u16) -> Option<WorkerInfo> {
+    //@todo I think wrapping this into it's own type would be nice just called WorkerList
+    pub async fn get_worker_list(&self) -> HashMap<u16, AgentWorkerInfo> {
+        self.worker_list.lock().await.clone()
+    }
+
+    pub async fn get_worker_by_session_id(&self, session_id: u16) -> Option<AgentWorkerInfo> {
         self.worker_list
             .lock()
             .await
@@ -558,7 +500,7 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
             .map(|w| w.clone())
     }
 
-    pub async fn update_worker_by_session_id(&self, session_id: u16, worker: WorkerInfo) {
+    pub async fn update_worker_by_session_id(&self, session_id: u16, worker: AgentWorkerInfo) {
         self.worker_list.lock().await.insert(session_id, worker);
     }
 
@@ -603,6 +545,10 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
         self.worker_info.lock().await.sid.clone()
     }
 
+    pub fn get_session_id(&self) -> u32 {
+        self.session_id
+    }
+
     // pub async fn set_sid(&self, sid: &str) {
     //     self.worker_info.lock().await.sid = Some(sid.to_owned());
     // }
@@ -647,12 +593,28 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
 
     pub async fn valid_share(&self) {
         self.stats.lock().await.accepted_shares += 1;
+        self.stats.lock().await.last_active = Utc::now().naive_utc();
         self.consider_ban().await;
         if *self.var_diff.lock().await {
             self.retarget().await;
         }
     }
 
+    pub async fn valid_share_by_session_id(&self, session_id: u16) {
+        match self.get_worker_by_session_id(session_id).await {
+            Some(worker) => {
+                worker.stats.lock().await.accepted_shares += 1;
+                worker.stats.lock().await.last_active = Utc::now().naive_utc();
+                //@todo we need to fix these to be worker specific.
+                self.consider_ban().await;
+                if *self.var_diff.lock().await {
+                    self.retarget().await;
+                }
+            }
+            None => {}
+        }
+    }
+
     //@todo I don't know if we actually need this function.
     // pub async fn get_next_difficulty(&self) -> Option<f64> {
     //     self.next_difficulty.lock().await.clone()
