@@ -1,4 +1,5 @@
 use crate::connection::Connection;
+use crate::id_manager::IDManager;
 use crate::router::Router;
 use crate::server::{UpstreamConfig, VarDiffConfig};
 use crate::types::{ExMessageGeneric, MessageValue};
@@ -16,7 +17,7 @@ use futures::io::{AsyncBufReadExt, AsyncReadExt};
 use futures::AsyncWriteExt;
 use futures::SinkExt;
 use futures::StreamExt;
-use log::{debug, info, trace, warn};
+use log::{debug, trace, warn};
 use serde_json::{Map, Value};
 use std::net::SocketAddr;
 
@@ -27,6 +28,7 @@ pub async fn handle_connection<
     State: Clone + Send + Sync + 'static,
     CState: Clone + Send + Sync + 'static,
 >(
+    id_manager: Arc<IDManager>,
     ban_manager: Arc<BanManager>,
     mut addr: SocketAddr,
     connection_list: Arc<MinerList<CState>>,
@@ -78,7 +80,16 @@ pub async fn handle_connection<
         //@todo name...
         let (mut urtx, urrx) = unbounded();
 
+        let connection_id = match id_manager.allocate_session_id().await {
+            Some(id) => id,
+            None => {
+                warn!("Sessions full");
+                return;
+            }
+        };
+
         let connection = Arc::new(Connection::new(
+            connection_id.clone(),
             addr,
             tx,
             utx,
@@ -234,6 +245,7 @@ pub async fn handle_connection<
 
         trace!("Closing stream from: {}", addr);
 
+        id_manager.remove_session_id(connection_id).await;
         connection_list.remove_miner(addr).await.unwrap();
 
         if connection.needs_ban().await {
